;;; zgrep.el --- M-x zgrep command to run "zgrep"

;; Copyright 2013, 2014, 2017 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: tools, processes, grep
;; URL: http://user42.tuxfamily.org/zgrep/index.html

;; zgrep.el is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; zgrep.el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.

;;; Commentary:
;; 
;; This is a simple couple of lines for an M-x zgrep to run the "zgrep"
;; program in the style of `M-x grep'.  See the `zgrep' docstring for more.

;;; Install:
;;
;; Put zgrep.el in one of your `load-path' directories and make `M-x zgrep'
;; available by adding to your .emacs
;;
;;     (autoload 'zgrep "zgrep" nil t)
;;
;; There's an autoload cookie for this if you install by
;; `M-x package-install' or know how to use `update-file-autoloads'.

;;; History:
;; 
;; Version 1 - the first version

;;; Code:

;;;###autoload
(defun zgrep ()
  "Run the \"zgrep\" command as per `M-x grep'.
`grep-command' is temporarily changed from \"grep\" to \"zgrep\".

`grep-apply-setting' is used to make the change, when available.

You can also simply `M-x grep' and edit the command offered, but
it's handy to have `M-x zgrep' do it.

See also `zrgrep'.

----
The zgrep.el home page is
URL `http://user42.tuxfamily.org/zgrep/index.html'"

  (interactive)
  (require 'grep)
  (grep-compute-defaults) ;; ensure `grep-command' value
  (let ((zgrep--old-grep-command grep-command))
    (unwind-protect
        (progn
          ;; `grep-apply-setting' new in emacs23
          (funcall (if (fboundp 'grep-apply-setting) 'grep-apply-setting 'set)
                   'grep-command (concat "z" grep-command))
          (call-interactively 'grep))

      (funcall (if (fboundp 'grep-apply-setting) 'grep-apply-setting 'set)
               'grep-command zgrep--old-grep-command))))

(provide 'zgrep)

;; LocalWords: el zgrep Ryde grep docstring

;;; zgrep.el ends here
