;; Copyright 2008, 2009, 2010, 2013, 2014, 2015 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


minibuffer-setup-hook
(completion-ignored-build-enable-setup)
completion-ignored-build-mode
minibuffer-complete
(load (expand-file-name "../completion-ignored-build.el"))
(completion-ignored-build-mode 1)

;;-----------------------------------------------------------------------------

(shell-command
 "mkdir /tmp/d-completion-ignored-build/;
  touch /tmp/d-completion-ignored-build/foo;
  chmod 755 /tmp/d-completion-ignored-build/foo;
  touch /tmp/d-completion-ignored-build/foo.c;
")
(progn
  (require 'completion-ignored-build)
  (switch-to-buffer (get-buffer-create "x"))
  (setq default-directory "/tmp/d-completion-ignored-build/")
  (insert (propertize "" 'field 'foo))
  (let ((completion-ignored-extensions completion-ignored-extensions))
    (completion-ignored-build-apply)
    (pp completion-ignored-extensions
        (current-buffer)))


;;-----------------------------------------------------------------------------
;; maybe

- ignore .dvi .log .out .pdf .pgf .ps .ps.gz from TeX
;;            
* Ignore foo.dvi, foo.log, foo.out, foo.pdf, foo.pgf (tikz/PGF),
  foo.ps, foo.ps.gz, foo.tpt,foo.tpm,foo.tno (thumbpdf) if
  there's any of foo.tex or foo.TeX (TeX), foo.ltx (LaTeX),
  foo.texi or foo.txi \(Texinfo).

        ;; foo.dvi .log .out .pgf .pdf .ps .ps.gz - generated from TeX
        ;; foo.tex, LaTeX foo.ltx, Texinfo foo.texi or foo.txi
        ;;
        ;; .pgf is from the PGF TeX package when running plain TeX or
        ;; ConTeXt, but not LaTeX.  Don't try to distinguish latex from
        ;; plain tex for this, just ignore .pgf for any tex.  Is .pgf used
        ;; by anything else?  Maybe could ignore it always anyway.
        ;;
        ;; Don't want to always exclude .dvi .pdf and .ps because .ps can be
        ;; hand-written and .ps .dvi and .pdf files can all be helpfully
        ;; visited with doc-view.el.  But if a foo.tex exists then that
        ;; source is is normally wanted and viewing its output can be done
        ;; with `tex-view' or similar.
        ;;
        ;; TeX generates various .aux .cp .toc too.  They're in the default
        ;; completion-ignored-extensions so ignored always already.
        ;;
        (and (string-match 
              "\\.\\(dvi\\|log\\|out\\|p[dg]f\\|ps\\(\\.gz\\)?\\)\\'" file)
             (let ((basename (substring file 0 (match-beginning 0))))
               ;; cf auto-mode-alist entries for tex-mode, latex-mode,
               ;; texinfo-mode
               (or (gethash (concat basename ".tex")  table)
                   (gethash (concat basename ".TeX")  table)
                   (gethash (concat basename ".ltx")  table)
                   (gethash (concat basename ".txi")  table)
                   (gethash (concat basename ".texi") table)))
             (add-to-list 'completion-ignored-extensions file))



;;-----------------------------------------------------------------------------

(unload-feature 'completion-ignored-build)
(require 'completion-ignored-build)
(ad-find-advice 'minibuffer-complete 'around 'completion-ignored-build)

(featurep 'completion-ignored-build)
(pp (load-history-filename-element "completion-ignored-build") (current-buffer))


(add-to-list 'completion-ignored-extensions
             "/tmp/foo")
(add-to-list 'completion-ignored-extensions
             "/foo")
(add-to-list 'completion-ignored-extensions
             "foo")
(my-remove-from-list 'completion-ignored-extensions
                     "foo")

;;-----------------------------------------------------------------------------

(defun completion-ignored-partition (func lst)
  (let (ret)
    (dolist (elem lst)
      (let* ((key (funcall func elem))
             (pair (assoc by-dir dir)))
        (if pair
            (push elem (cdr pair))
          (push (cons key (list elem)) ret))))
    ret))

(defun completion-ignored-build-dirhash (dir)
  (let ((file-list (condition-case nil ;; no error if no such dir
                       (directory-files dir-name) ;; relative filenames
                     (error nil)))
        (table     (make-hash-table :size (* 3 (length file-list))
                                    :test 'equal)))
    (dolist (file file-list)
      (puthash file t table))
    table))

(defun completion-ignored-build-hash (elem)
  (if (hash-table-p (car elem))
      (car elem)
    (setcar elem (completion-ignored-build-dirhash (car elem)))))

(defun completion-ignored-build-files (file-list)
  (let ((by-dir (completion-ignored-partition 'file-name-directory file-list)))
    (dolist (elem by-dir)
      (let ((default-directory (car elem))
            (file-list         (cdr elem)))

        ;; Makefile.in - generated by automake from Makefile.am
        (and (gethash "Makefile.in" table)
             (gethash "Makefile.am" table)
             (add-to-list 'completion-ignored-extensions "Makefile.in"))

        ))))
        

(defun completion-ignored-build-apply ()
  (let ((minibuf (field-string)))
    (setq minibuf (substitute-in-file-name minibuf))

    (let* ((dir (file-name-directory minibuf))
           (default-directory (if dir
                                  (expand-file-name dir)
                                default-directory))
           (file-list (condition-case nil ;; no error if no such dir
                          (directory-files dir) ;; relative filenames
                        (error nil))))
      (setq completion-ignored-extensions
            (completion-ignored-build-files file-list)))))

(defun completion-ignored-build-files (file-list)
  (let (ret)
    (dolist (filename file-list)
      (let ((basename (file-name-nondirectory filename))
            (default-directory (expand-file-name
                                (file-name-directory filename))))

        (if (or
             ;; Makefile - variously generated
             (and (equal basename "Makefile")
                  (or (file-exists-p "Makefile.am")   ;; automake
                      (file-exists-p "Makefile.in")   ;; generic autoconf
                      (file-exists-p "Makefile.PL"))) ;; perl

             ;; Makefile.in - generated by automake from Makefile.am
             (and (equal basename "Makefile.in")
                  (file-exists-p "Makefile.am"))

             ;; Makefile.old -- generated by perl MakeMaker (a backup)
             (and (equal basename "Makefile.old")
                  (file-exists-p "Makefile.PL"))

             ;; configure -- generated by autoconf configure.in or configure.ac
             (and (equal basename "configure")
                  (or (file-exists-p "configure.in")
                      (file-exists-p "configure.ac")))

             ;; Build -- generated by perl Module::Build from Build.PL
             (and (equal basename "Build")
                  (file-exists-p "Build.PL"))

             ;; foo-N, foo.info-N, foo-N.gz and foo.info-N.gz -- info sub-files
             (and (string-match "-[0-9]+\\(\\.gz\\)\\'" basename)
                  (file-exists-p (concat (substring file 0 (match-beginning 0))
                                         (match-string 1 file))))

             ;; foo.c - generated by perl xsubpp from foo.xs
             (and (string-match "\\.c\\'" file)
                  (file-exists-p (concat (substring file 0 (match-beginning 0))
                                         ".xs")))

             ;; foo.info - generated by makeinfo from foo.texi
             (and (string-match "\\.info\\'" file)
                  (file-exists-p (concat (substring file 0 (match-beginning 0))
                                         ".texi"))))

            (push filename ret))

        ;; WRONG: .c may not be in file-list

        ;; foo - executable from foo.c, foo.cc or foo.C program
        (and (string-match "\\.\\(cc?\\|C\\)\\'" basename)
             (>= (match-beginning 0) 3)
             (let ((built (substring file 0 (match-beginning 0))))
               (member built file-list)
               (push built ret)))

        ))
    ret))
        

(setq lst (directory-files "/doc"))
(dolist (file lst)
  (file-exists-p (concat file ".c"))
  (file-exists-p (concat file ".cc"))
  (file-exists-p (concat file ".C")))


;;-----------------------------------------------------------------------------
;; advice.el dependency

(progn
  ;; (load "advice")
  (add-to-list 'load-path "../wip")
  (load "completion-ignored-build")
  (unload-feature 'advice)
  (unload-feature 'completion-ignored-build))


;;-----------------------------------------------------------------------------
;; hash

(let* ((file-list (directory-files "/doc"))
      (table (make-hash-table :size (length file-list)
                              :test 'equal)))
  (dolist (file file-list)
    (puthash file t table))
  (dolist (file file-list)
    (gethash (concat file ".c") table)))

(let* ((file-list (directory-files "/doc")))
  (dolist (file file-list)
    (member (concat file ".c") file-list)))


;;-----------------------------------------------------------------------------

(message "completion-ignored-build minibuf=%S" minibuf)

(expand-file-name "scp:1" "/")
(expand-file-name "/")
(expand-file-name "scp:1" "/foo/bar/")
(expand-file-name "scp:1" "/tmp/")


(progn
  (defun my-show-overlay-lists ()
    "Kill the current buffer."
    (interactive)
    (message "overlay-lists %S" (overlay-lists))
    (message "overlay-properties %S"
             (mapcar 'overlay-properties (remq nil (overlay-lists))))
    (message "field-beginning %S field-end %S substring %S"
             (field-beginning) (field-end)
             (buffer-substring (field-beginning) (field-end)))
    (message "field-string %S"
             (field-string (point-max)))
    (message "minibuffer-completion-table %S"
             minibuffer-completion-table))
  (define-key global-map [f10] 'my-show-overlay-lists))

(expand-file-name "/tmp//etc")
(file-name-completion "/tmp//etc" "/")
(debug-on-entry 'file-name-completion)
(cancel-debug-on-entry 'file-name-completion)

;;-----------------------------------------------------------------------------

(ad-disable-advice 'minibuffer-complete 'around 'completion-ignored-build)
(ad-activate 'minibuffer-complete)
(completion-ignored-build-enable)
(completion-ignored-build-disable)

(dolist (func completion-ignored-build--advised-functions)
  (ad-disable-advice func 'around 'completion-ignored-build)
  (ad-activate func))

(debug-on-entry 'ad-activate)
(cancel-debug-on-entry 'ad-activate)
(pp (symbol-function 'minibuffer-complete) (current-buffer))


;;-----------------------------------------------------------------------------

(progn
  (add-to-list 'completion-ignored-extensions ".old")
  (add-to-list 'completion-ignored-extensions ".bs"))

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (add-to-list 'load-path (expand-file-name ".."))
  (autoload 'completion-ignored-build-enable "completion-ignored-build" nil t)
  (add-hook 'minibuffer-setup-hook 'completion-ignored-build-enable))

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (add-to-list 'load-path (expand-file-name ".."))
  (require 'completion-ignored-build))

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (add-to-list 'load-path (expand-file-name ".."))
  (require 'completion-ignored-build)
  (customize-variable 'minibuffer-setup-hook))

(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (add-to-list 'load-path (expand-file-name ".."))
  (add-hook 'minibuffer-setup-hook
	    (lambda ()
	      (require 'completion-ignored-build))))

(setq debug-on-error t)


;;-----------------------------------------------------------------------------
;; icicles bug

(progn
  (defadvice minibuffer-complete-and-exit (before my-bit activate)
    "Something more for minibuffer exiting."
    (message "hi"))
  (ad-deactivate 'minibuffer-complete-and-exit)
  (require 'icicles)
  (ad-activate 'minibuffer-complete-and-exit)
  (completing-read "foo" '("x") nil t))

(progn
  (defadvice minibuffer-complete-and-exit (before my-bit activate)
    "Something more for minibuffer exiting."
    (message "hi"))
  (require 'icicles)
  (ad-activate 'minibuffer-complete-and-exit)
  (completing-read "foo" '("x") nil t))

(progn
  (defadvice minibuffer-complete-and-exit (before my-bit activate)
    "Something more for minibuffer exiting."
    (message "hi"))
  (require 'icicles)
  (completing-read "foo" '("x") nil t))

(describe-function 'minibuffer-complete-and-exit)

(progn
  (defun foo ()
    (message "foo"))
  '(defadvice foo (before more-foo activate)
    "Something more for foo."
    (message "more foo"))

  (fset 'old-foo (symbol-function 'foo))
  (defalias 'foo 'old-foo)

  (foo))

  (describe-function 'old-foo)
  (describe-function 'foo))
(symbol-function 'foo)
(symbol-function 'old-foo)


(defun bar ()
  (message "replaced foo"))
(defalias 'foo 'bar)

(progn
  (gpm-mouse-mode 0)
  (require 'icicles)
  (setq debug-on-error t)
  (debug-on-entry 'ad-make-advised-docstring)
  (describe-function 'minibuffer-complete-and-exit)
  )
(require 'completion-ignored-build)
icicle-candidate-help-fn

/usr/share/emacs/site-lisp/icicles

;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))


;;-----------------
;; xemacs minibuffer-contents-no-properties

(progn
  (define-key minibuffer-local-map [f8] 'my-try-minibuffer)
  (defun my-try-minibuffer ()
    (interactive)
    (let ((str (buffer-string)))
      (message "'%s'" str)))
  (read-from-minibuffer "Prompt: " "initial content"))

(defun completion-ignored-build--minibuffer-contents-no-properties ()
  "Return the minibuffer contents, with no text properties.
This is `minibuffer-contents-no-properties' in Emacs, or a
fallback in XEmacs 21."
  (if (eval-when-compile (fboundp 'minibuffer-contents-no-properties))
      (minibuffer-contents-no-properties)   ;; emacs21 up
    ;; xemacs21 -- prompt isn't actual buffer contents
    (buffer-substring-no-properties (point-min) (point-max))))


;;-----------------



(ad-enable-advice 'minibuffer-complete 'around 'dynamic-completion-ignored)

  (ad-enable-regexp "\\`completion-ignored-build\\'"))
  (ad-disable-regexp "\\`completion-ignored-build\\'")

(macroexpand '(defadvice minibuffer-complete (around dynamic-completion-ignored
                                       activate)
  "Dynamic `completion-ignored-extensions'."
  (if minibuffer-completing-file-name
      (let ((completion-ignored-extensions completion-ignored-extensions))
        (dynamic-completion-ignored-apply)
        ad-do-it)
    ad-do-it)))
(progn 
  (ad-add-advice (quote minibuffer-complete)
                 (quote (dynamic-completion-ignored nil t ...))
                 (quote around)
                 (quote nil))
  (ad-activate (quote minibuffer-complete) nil)
  (quote minibuffer-complete)
  )

(progn (ad-add-advice (quote minibuffer-complete) (quote (dynamic-completion-ignored nil t (advice lambda nil "Dynamic `completion-ignored-extensions'." (if minibuffer-completing-file-name (let ((completion-ignored-extensions completion-ignored-extensions)) (dynamic-completion-ignored-apply) ad-do-it) ad-do-it)))) (quote around) (quote nil))
       (ad-activate (quote minibuffer-complete)
                    nil)
       (quote minibuffer-complete))

(defadvice minibuffer-complete-word (around dynamic-completion-ignored
                                            activate)
  "Dynamic `completion-ignored-extensions'."
  (if minibuffer-completing-file-name
      (let ((completion-ignored-extensions completion-ignored-extensions))
        (dynamic-completion-ignored-apply)
        ad-do-it)
    ad-do-it))

(defadvice minibuffer-complete-and-exit (around dynamic-completion-ignored
                                                activate)
  "Dynamic `completion-ignored-extensions'."
  (if minibuffer-completing-file-name
      (let ((completion-ignored-extensions completion-ignored-extensions))
        (dynamic-completion-ignored-apply)
        ad-do-it)
    ad-do-it))










;; (defadvice file-name-all-completions (after my-ignore activate)
;;   "Find again without <1> etc put by makeinfo, if necessary."
;;   (debug)
;;   (message "xx"))
;; 
;; (file-name-all-completions "" "/tmp")
;; 
;; (defadvice file-name-completion (after my-ignore activate)
;;   "Find again without <1> etc put by makeinfo, if necessary."
;;   (debug)
;;   (message "xx"))


;; (defvar dynamic-completion-ignored nil)
;; 
;; (defadvice read-file-name (around dynamic-completion-ignored activate)
;;   (let ((dynamic-completion-ignored t))
;;     ad-do-it))


;; (read-file-name "jfksd ")


;;-----------------------------------------------------------------------------
;; unused

(defun completion-ignored-build--any-suffix-p (lst suffix)
  "Return non-nil if any string in LST ends with SUFFIX."

  ;; Using cl.el this would be
  ;; (find (concat "." (regexp-quote suffix) "\\'") lst :test `string-match))
  ;;
  ;; but want to avoid cl since its load message can be a bit irritating
  ;; during completion, and since a filename completion is done so often you
  ;; would end up dragging in cl almost immediately
  ;;
  (let ((regexp (concat "." (regexp-quote suffix) "\\'"))
        found)
    (while (and lst (not found))
      (if (string-match regexp (car lst))
          (setq found t)
        (setq lst (cdr lst))))
    found))

;;     (dolist (elem '((t   ".xs" ("foo.xs" "foo.c"))
;;                     (t   ".xs" ("foo.c" "foo.xs"))
;;                     (t   ".c"  ("foo.c" "foo.xs"))
;;                     (nil ".xs" ())
;;                     (nil ".xs" ("foo.c"))
;;                     (t   ".c"  ("foo.c"))
;;                     (nil ".xs" ("fooxs"))
;;                     ;; not a bare dot file
;;                     (nil ".xs" (".xs"))))
;; 
;;       (let* ((want   (nth 0 elem))
;;              (suffix (nth 1 elem))
;;              (lst    (nth 2 elem))
;;              (got    (not (not (completion-ignored-build--any-suffix-p lst suffix)))))
;;         (unless (equal want got)
;;           (error "wrong at %s suffix %s lst %s\nwant %s\ngot  %s"
;;                  count suffix lst want got)))
;;       (setq count (1+ count)))

