;; Copyright 2009, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(defun my-urls ()
  (find-file "/tmp/emacs-urls.html")
  (erase-buffer)
  (insert "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">
<html>
<head>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">
<title>Emacs Links</title>
</head>
<body>

")

  (my-urls-docstring-urls)
  (my-urls-custom-url-links)
  (my-urls-more-stuff)
  (my-urls-newsticker)

  (insert "\n</body>\n</html>"))


(defun my-urls-more-stuff ()
  (let ((outbuf   (current-buffer))
        (filename "/down/emacs/etc/MORE.STUFF"))
    (save-current-buffer
      (let ((enable-local-variables nil))
        (find-file filename))

      (goto-char (point-min))
      (while (re-search-forward "<URL:\\([^>]+\\)" nil t)
        (let ((url     (match-string 1))
              (linenum (line-number-at-pos (match-beginning 0))))
          (with-current-buffer outbuf
            (insert (format "<p> %s
<br> line %s
<br> <a href=\"%s\">%s</a>
</p>

" filename linenum url url)))))

      (bury-buffer))))


(defun my-urls-newsticker ()
  (require 'newsticker)
  (dolist (elem newsticker--raw-url-list-defaults)
    (let ((name (nth 0 elem))
          (url  (nth 1 elem)))
      (insert (format "<p> newsticker--raw-url-list-defaults
<br> \"%s\"
<br> <a href=\"%s\">%s</a>
</p>

" name url url)))))


(defun my-urls-custom-url-links ()
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-load-all)
  (my-load-all--custom)

  (mapatoms
   (lambda (symbol)
     (dolist (link (get symbol 'custom-links))
       (when (eq 'url-link (car link))
         ;; skip :tag part
         (if (eq :tag (cadr link))
             (setq link (cddr link)))

         (let ((url  (cadr link))
               (file (symbol-file symbol)))
           (unless (string-match "^mailto:" url )
             (insert (format "<p> customize url-link
<br> `%s'
<br> file %s
<br> <a href=\"%s\">%s</a>
</p>

" symbol file url url)))))))))


(defun my-urls-docstring-urls ()
;;   (add-to-list 'load-path (expand-file-name "."))
;;   (require 'my-load-all)
;;   (my-load-all--custom)


  (let ((case-fold-search nil)
        (outbuf           (current-buffer)))
    (mapatoms
     (lambda (symbol)
       (when (and (not (eq 'calendar-mode symbol))
                  (fboundp symbol)
                  (symbol-function symbol)) ;; not nil
         (let ((str (condition-case nil
                        (documentation symbol)
                      (error nil))))
           (when str
             (with-temp-buffer
               (insert str)
               (goto-char (point-min))
               (while (re-search-forward "URL `\\([^']+\\)'" nil t)
                 (let ((url  (match-string 1))
                       (file (symbol-file symbol)))
                   (with-current-buffer outbuf
                     (insert (format "<p> docstring URL
<br> `%s'
<br> file %s
<br> <a href=\"%s\">%s</a>
</p>

" symbol file url url)))))))))))))



;;   (apply 'call-process
;;          "linkchecker" nil info-xref-output-buffer t
;;          "-r0" "--no-status" url-links)))


