;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count 0)
        (case-fold-search nil)
        (tempbuf "*t-checkdoc-autoload.el temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)


    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(checkdoc-autoload-comment))

    ;; only a defvar, so no custom options yet
    ;;
    ;; (mapcar (lambda (elem)
    ;;           (let* ((hook (nth 0 elem))
    ;;                  (func (nth 1 elem))
    ;;                  (options (get hook 'custom-options)))
    ;;             (unless (and (listp options)
    ;;                          (memq func options))
    ;;               (error "`%S' not an option on `%S'" func hook)))
    ;;           (setq count (1+ count)))
    ;;
    ;;         '((checkdoc-comment-style-hooks checkdoc-autoload-defgroup)
    ;;           (checkdoc-comment-style-hooks checkdoc-autoload-puts)
    ;;           (checkdoc-comment-style-hooks checkdoc-autoload-entrypoint)
    ;;           (checkdoc-comment-style-hooks checkdoc-autoload-comment)))


    ;;-------------------------------------------------------------------------
    ;; checkdoc-autoload-put-p

    (require 'checkdoc-autoload)

    (let ((checkdoc-autofix-flag 'automatic)
          (checkdoc-style-hooks '(checkdoc-autoload-docstring)))
      (mapcar (lambda (elem)
                (let* ((want (nth 0 elem))
                       (form (nth 1 elem))
                       (got  (checkdoc-autoload-put-p form)))
                  (unless (equal want got)
                    (error "checkdoc-autoload-put-p\nform %S\ngot  %S\nwant %S"
                           form got want)))
                (setq count (1+ count)))

              '((risky-local-variable
                 (put 'something 'risky-local-variable t))

                (safe-local-variable
                 (put 'something 'safe-local-variable 'predicate))

                (nil ())
                (nil 123)
                (nil "x")
                (nil (123))
                (nil ("x"))
                (nil (put))
                (nil (put . 123))
                (nil (put . "x"))
                (nil (put nil))
                (nil (put 'var))
                (nil (put 'var . 123))
                (nil (put 'var . "x"))
                (nil (put 'var . "x"))

                ;; validation doesn't extend past third element, so these pass
                (risky-local-variable
                 (put 'var 'risky-local-variable))
                (risky-local-variable
                 (put 'var 'risky-local-variable . 123))
                (risky-local-variable
                 (put 'var 'risky-local-variable . "x")))))


    ;;-------------------------------------------------------------------------
    ;; checkdoc-autoload-interactive-defun-p

    (require 'checkdoc-autoload)

    (let ((checkdoc-autofix-flag 'automatic)
          (checkdoc-style-hooks '(checkdoc-autoload-docstring)))
      (mapcar (lambda (elem)
                (let* ((want (nth 0 elem))
                       (form (nth 1 elem))
                       (got (checkdoc-autoload-interactive-defun-p form)))
                  (unless (equal want got)
                    (error "checkdoc-autoload-interactive-defun-p %S got %S want %S"
                           form got want)))
                (setq count (1+ count)))

              '((t   (defun foo () (interactive)))
                (t   (defun foo () "" (interactive)))
                (nil (defun . ""))
                (nil (defun "" . 999))
                (nil (defun foo "" . 999))
                (nil (defun foo "" (bar))))))


    ;;-------------------------------------------------------------------------
    ;; checkdoc-autoload-interactive-p

    (require 'checkdoc-autoload)

    (let ((checkdoc-autofix-flag 'automatic)
          (checkdoc-style-hooks '(checkdoc-autoload-docstring)))
      (mapcar (lambda (elem)
                (let* ((want (nth 0 elem))
                       (form (nth 1 elem))
                       (got (checkdoc-autoload-interactive-p form)))
                  (unless (equal want got)
                    (error "checkdoc-autoload-interactive-p %S got %S want %S"
                           form got want)))
                (setq count (1+ count)))

              '((t  (defun foo () (interactive)))
                (t  (defun foo () "" (interactive)))

                (t  (define-derived-mode foo-mode))
                (t  (define-generic-mode foo-mode))
                (t  (define-minor-mode  foo-mode))
                (t  (define-globalized-minor-mode foo-mode))
                (t  (easy-mmode-define-global-mode foo-mode))
                (t  (easy-mmode-define-minor-mode  foo-mode))

                (nil  (defun . ""))
                (nil  (defun "" . 999))
                (nil  (defun foo "" . 999))
                (nil  (defun foo "" (bar))))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (require 'checkdoc-autoload)
    (unload-feature 'checkdoc-autoload t)

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
