;; Copyright 2006, 2007, 2008, 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count   0)
        (tempbuf "*t-compilation-onewin--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(compilation-onewin-mode))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'compilation-onewin)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; clone-buffer

    (if (not (fboundp 'clone-buffer))
        (message "skip, no clone-buffer in this Emacs")

      ;; works
      (let* ((buf   (get-buffer-create "*t-compilation-onewin clone*"))
             (clone (with-current-buffer buf (clone-buffer))))
        (kill-buffer buf)
        (kill-buffer clone)
        (setq count (1+ count)))

      ;; carries hash setting
      (let ((buf (get-buffer-create "*t-compilation-onewin clone*")))
        (puthash buf t compilation-onewin-buffer-hash)

        (let ((clone (with-current-buffer buf (clone-buffer))))
          (unless (gethash buf compilation-onewin-buffer-hash)
            (error "clone-buffer loses original hash setting"))
          (unless (gethash clone compilation-onewin-buffer-hash)
            (error "clone-buffer hash setting not carried across"))

          (kill-buffer buf)
          (kill-buffer clone)
          (setq count (1+ count)))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'compilation-onewin t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'compilation-onewin))
        (require feature)
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'compilation-onewin))
        (require feature)
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; clone-buffer still works after unload

    (if (not (fboundp 'clone-buffer))
        (message "skip, no clone-buffer in this Emacs")

      (let* ((buf   (get-buffer-create "*t-compilation-onewin clone*"))
             (clone (with-current-buffer buf (clone-buffer))))
        (kill-buffer buf)
        (kill-buffer clone)
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; compilation-start still works after unload

    (if (not (fboundp 'compilation-start))
        (message "skip, no compilation-start in this Emacs")

      (let ((buf (compilation-start "true")))
        (with-current-buffer buf
          (if (get-buffer-process buf)
              (kill-compilation))
          (sit-for .5)
          (kill-buffer nil)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; compile-internal still works after unload

    (if (not (fboundp 'compile-internal))
        (message "skip, no compile-internal in this Emacs")

      (let ((buf (compile-internal "true" "no more errors")))
        (with-current-buffer buf
          (if (get-buffer-process buf)
              (kill-compilation))
          (sit-for .5)
          (kill-buffer nil)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
