;; Copyright 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-completion-ignored-build--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(completion-ignored-build-mode
              completion-ignored-build-enable))


    ;;-------------------------------------------------------------------------
    ;; load

    (require 'completion-ignored-build)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; mode on/off

    ;; initial M-x completion-ignored-build-mode
    (completion-ignored-build-mode)
    (unless completion-ignored-build-mode
      (error "completion-ignored-build-mode: initial load not enabled"))

    (completion-ignored-build-mode 1)
    (unless completion-ignored-build-mode
      (error "completion-ignored-build-mode 1: not enabled"))
    (setq count (1+ count))

    (completion-ignored-build-mode 0)
    (when completion-ignored-build-mode
      (error "completion-ignored-build-mode toggle: not disabled"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; advice enable/disable

    (completion-ignored-build-enable)
    (unless (ad-advice-enabled
             (ad-find-advice 'minibuffer-complete 'around
                             'completion-ignored-build))
      (error "completion-ignored-build-enable: advice not applied"))
    (setq count (1+ count))

    (completion-ignored-build-disable)
    (when (ad-advice-enabled
           (ad-find-advice 'minibuffer-complete 'around
                           'completion-ignored-build))
      (error "completion-ignored-build-disable: advice not removed"))
    (setq count (1+ count))

    (completion-ignored-build-disable)
    (when (ad-advice-enabled
           (ad-find-advice 'minibuffer-complete 'around
                           'completion-ignored-build))
      (error "completion-ignored-build-disable twice: advice not removed"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'completion-ignored-build))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'completion-ignored-build))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
