;; Copyright 2009, 2010, 2011, 2013, 2017 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-string-make-unibyte (str)
  (if (fboundp 'string-make-unibyte)
      (string-make-unibyte str)
    str))

(defconst my-unicode-bullet-str
  (or (condition-case nil
          (string (decode-char 'ucs 8226)) ;; U+2022
        (error nil))
      (condition-case nil
          (decode-coding-string (my-string-make-unibyte (string 226 128 162))
                                'utf-8)
        (error nil))))

(defun my-run-test ()

  (let ((count 0)
        (tempbuf "*t-ffap-pod-F--temp*")
        (case-fold-search t))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (require 'miscbits-el-loaddefs)

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(ffap-pod-F-enable))

    (mapcar (lambda (func)
              (unless (commandp func)
                (error "`%S' not interactive" func))
              (setq count (1+ count)))
            '(ffap-pod-F-enable))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap)
    (require 'ffap-pod-F)

    ;; no leftovers in emacs21 byte compiling defadvice
(when (boundp 'byte-compile-unresolved-functions)
      (setq byte-compile-unresolved-functions
            (delete '(thing-at-point-looking-at 1) ;; loaded dynamically
                    byte-compile-unresolved-functions))
      (when byte-compile-unresolved-functions
        ;; (message "unr %S" byte-compile-unresolved-functions)
        ;; (message "fb %S" (fboundp (caar byte-compile-unresolved-functions)))
        (error "byte-compile-unresolved-functions after load: %S"
               byte-compile-unresolved-functions)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- when never enabled

    (ad-activate 'ffap-string-at-point)

    (erase-buffer)

    (let ((got  (ffap-string-at-point))
          (want ""))
      (unless (equal got want)
        (error "ffap-string-at-point in empty buffer got %S want %S"
               got want))
      (setq count (1+ count)))

    (let ((got  (ffap-file-at-point))
          (want nil))
      (unless (equal got want)
        (error "ffap-file-at-point in empty buffer got %S want %S"
               got want))
      (setq count (1+ count)))


    (erase-buffer)
    (insert "F</>")
    (goto-char (point-min))

    (let ((got  (ffap-string-at-point))
          (want "F</"))
      (unless (equal got want)
        (error "ffap-string-at-point when never enabled got %S want %S"
               got want))
      (setq count (1+ count)))

    (let ((got  (ffap-file-at-point))
          (want nil))
      (unless (equal got want)
        (error "ffap-file-at-point when never enabled got %S want %S"
               got want))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- enabled

    (erase-buffer)
    (insert "F</tmp>")
    (goto-char (point-min))

    (ffap-pod-F-enable)

    (let ((got  (ffap-string-at-point))
          (want "/tmp"))
      (unless (equal got want)
        (error "ffap-string-at-point when enabled got: %S" got))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; ffap-pod-F-ucs-string

    (mapcar (lambda (elem)
              (let* ((num  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (ffap-pod-F-ucs-string num)))
                (unless (equal got want)
                  (error "ffap-pod-F-ucs-string %S got %S want %S"
                         num got want))
                (setq count (1+ count))))

            `((88  "X")
              (247 ;; division sign
               ,(decode-coding-string (string 247) 'iso-8859-1))))

    (if (null (ffap-pod-F-ucs-string 8226))
        (message "skip, no unicode decode")

      (mapcar (lambda (elem)
                (let* ((num  (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (ffap-pod-F-ucs-string num)))
                  (unless (equal got want)
                    (error "ffap-pod-F-ucs-string %S got %S[%s] want %S[%s]"
                           num
                           got (length got)
                           want (length want)))
                  (setq count (1+ count))))

              `((8226 ,my-unicode-bullet-str))))  ;; bullet


    ;;-------------------------------------------------------------------------
    ;; ffap-pod-F-string-to-number

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (ffap-pod-F-string-to-number str)))
                (unless (equal got want)
                  (error "ffap-pod-F-string-to-number %S got %S want %S"
                         str got want))
                (setq count (1+ count))))

            `(("123"  123)
              ("0377" 255)
              ("0" 0)
              ("00" 0)
              ("0x0" 0)
              ("0X00" 0)
              ("0xA0" 160)
              ("0xa" 10)))

    ;;-------------------------------------------------------------------------
    ;; ffap-pod-F-unescape

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (ffap-pod-F-unescape str)))
                (unless (equal got want)
                  (error "ffap-pod-F-unescape %S got %S want %S"
                         str got want))
                (setq count (1+ count))))

            `(("E<lt>"  "<")
              ("E<gt>Z<>" ">")
              ("E<verbar>E<sol>" "|/")
              ("E<verbar>abcE<sol>" "|abc/")
              ("E<0x41>E<66> " "AB ")
              ("\tE<0x4B>E<0x6c>" "\tKl")))


    ;;-------------------------------------------------------------------------
    ;; ffap-pod-F-at-point

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))

                (let ((got (ffap-pod-F-at-point)))
                  (unless (equal got want)
                    (error "ffap-pod-F-at-point %S got %S want %S"
                           str got want)))
                (setq count (1+ count))))

            `(("F<foo>"  "foo")
              ("F<~E<sol>barE<sol>quux>"  "~/bar/quux")
              ("F<< abc >>"  "abc")
              ("F<<< def/ghi >>>"  "def/ghi")))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-pod-F))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-pod-F))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- still works when unloaded

    (erase-buffer)
    (let ((got  (ffap-string-at-point))
          (want ""))
      (unless (equal got want)
        (error "ffap-string-at-point in empty buffer after unload got: %S"
               got))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
