;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search nil))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'make-mode-fillindent)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (let* ((feature 'make-mode-fillindent)
           (adv     (my-advice-by-name feature)))

      (unload-feature feature t)

      (cond ((not adv)
             (message "skip, no defadvice in this emacs"))
            ((featurep 'xemacs)
             (message "skip, xemacs doesn't unload-hook"))
            (t
             (let ((lst (my-advice-by-name feature)))
               (when lst
                 (error "unload-feature leftover advice: %S" lst))
               (setq count (1+ count))))))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'make-mode-fillindent))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

