;; miscbits-generate-finder.el -- create miscbits-el-finder-inf.el file



;; finder-inf: miscbits-el-finder-inf.el
;; miscbits-el-finder-inf.el: $(EL_FILES_COMPILE) tools/miscbits-generate-finder.el
;; 	$(EMACS) -q -no-site-file -batch \
;; 	  -l tools/miscbits-generate-finder.el \
;; 	  -f my-batch-generate-finder \
;; 	  `pwd`
;; maintainer-clean: clean
;; 	rm -f miscbits-el-finder-inf.el

;; | grep -v miscbits-el-finder-inf | wc -l


;; Copyright 2009, 2010, 2011, 2015 Kevin Ryde.

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Keywords: miscbits-el
;; URL: http://user42.tuxfamily.org/miscbits-el/index.html

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Usage: emacs -batch -q -no-site-file \
;;          -l miscbits-generate-finder.el \
;; 	    -f my-batch-generate-finder \
;;          DIR
;;
;; Create file miscbits-el-finder-inf.el from *.el files in the given DIR,
;; so those files appear under `finder-by-keyword'.
;;
;; Suspect this might only work properly in emacs23.

;;; Code:

(defvar miscbits-generate-finder--running nil)

(defadvice find-file (before miscbits-generate-finder activate)
  "Hack for `generated-finder-keywords-file' in xemacs21."
  (and miscbits-generate-finder--running
       (equal filename "finder-inf.el")
       (ad-set-arg 0 generated-finder-keywords-file)))

(defun my-generate-finder ()
  (require 'finder)

  (setq generated-finder-keywords-file "miscbits-el-finder-inf.el")
  (condition-case nil
      (delete-file generated-finder-keywords-file)
    (error nil))

  (find-file generated-finder-keywords-file)
  (setq buffer-read-only nil)

  (let ((miscbits-generate-finder--running t))
    (finder-compile-keywords "."))

  (goto-line 2)
  (if (looking-at ";;.*")
      (replace-match ""))
  (insert "
;; Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2015 Kevin Ryde.

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Keywords: miscbits-el
;; URL: http://user42.tuxfamily.org/miscbits-el/index.html

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.
")

  (search-forward ";;; Code:\n")
  (insert "\n;;;###autoload (eval-after-load \"finder\" '(require 'miscbits-el-finder-inf))\n(require 'finder)\n")

  ;; Wrap in `append' to builtin `finder-package-info'.
  ;; setq in emacs, defconst in xemacs
  (re-search-forward "(\\(setq\\|defconst\\) finder-package-info")
  (replace-match "
 (let (extra)
   (mapcar (lambda (elem)
             (if (not (assoc (car elem) finder-package-info))
                 (setq extra (cons elem extra))))
")

  (re-search-forward "))\n\\(\f\\|\n(provide\\)")
  (goto-char (match-beginning 0))
  (insert "))
 (setq finder-package-info (append finder-package-info extra)
")

  ;; Don't suppress autoload generation, so can pick up cookie inserted
  ;; above.  Check if present though, since xemacs21 doesn't emit this.
  (goto-char (point-min))
  (when (re-search-forward "^;+ no-update-autoloads: t\n"
                           nil t) ;; not generated by xemacs21
    (replace-match ""))

  ;; Should be all ascii.
  (goto-char (point-min))
  (when (re-search-forward "[^\000-\177]" nil t)
    (error "Oops, non ascii in generated finder-inf"))

  ;; Remove utf-8 coding since no such coding system in xemacs21, and file
  ;; is all ascii anyway.  Check if present though, since xemacs21 doesn't
  ;; emit it.
  (goto-char (point-min))
  (when (re-search-forward "^;+ coding: utf-8\n"
                           nil t) ;; not generated by xemacs21
    (replace-match ""))

  (goto-char (point-min))
  (save-buffer))

(defun my-batch-generate-finder ()
  (let ((default-directory (car command-line-args-left)))
    (my-generate-finder))
  (kill-emacs 0))

(provide 'miscbits-generate-finder)

;;; miscbits-generate-finder.el ends here
